@@ -178,7 +178,7 @@ pd_update_general_tab (EogPropertiesDialog *prop_dlg,
 
 #if HAVE_EXIF
 static void
-eog_exif_set_label (GtkWidget *w, EogExifData *exif_data, gint tag_id)
+eog_exif_set_label (GtkWidget *w, ExifData *exif_data, gint tag_id)
 {
 	gchar exif_buffer[512];
 	const gchar *buf_ptr;
@@ -781,6 +781,17 @@ eog_properties_dialog_init (EogPropertiesDialog *prop_dlg)
 #endif
 }
 
+/**
+ * eog_properties_dialog_new:
+ * @parent: 
+ * @thumbview: 
+ * @next_image_action: 
+ * @previous_image_action: 
+ *
+ * 
+ *
+ * Returns: (transfer full): a new #EogPropertiesDialog
+ **/
 GObject *
 eog_properties_dialog_new (GtkWindow    *parent,
 			   EogThumbView *thumbview,
